@@ -1,7 +1,8 @@
-use corundum::alloc::*;
+use corundum::*;
 use std::env;
+use corundum::open_flags::*;
 
-type P = corundum::default::BuddyAlloc;
+type P = corundum::default::Allocator;
 
 fn main() {
     let args: Vec<String> = env::args().collect();
