@@ -9,6 +9,7 @@ mod stack;
 use consumer::Consumer;
 use hashmap::*;
 use corundum::default::*;
+use corundum::open_flags::*;
 use producer::Producer;
 use stack::*;
 use std::env;
@@ -17,7 +18,7 @@ use std::io::{BufRead, BufReader};
 use std::thread;
 
 pub static mut PRINT: bool = true;
-type P = BuddyAlloc;
+type P = Allocator;
 
 fn help() {
     println!("usage: grep [OPTIONS] list-file");
@@ -30,7 +31,7 @@ fn help() {
     println!("  -N            Do not print output (perf test)");
     println!("  -D            Distribute text partitions to consumers (used with -I)");
     println!("  -I            Continue counting in isolation (used with -D)");
-    println!("  -C            Continue from the previous run (used with -)P");
+    println!("  -C            Continue from the previous run (used with -P)");
     println!("  -P            Prepare only (do not run threads, used with -C)");
     println!("  -h            Display help");
     println!();
@@ -126,7 +127,7 @@ fn main() {
         }
     }
     
-    let root = P::open::<Root>(&pool, O_CFNE | O_8GB).unwrap();
+    let root = P::open::<Root>(&pool, O_CFNE | O_1GB).unwrap();
 
     P::transaction(|j| {
         let mut producers = root.producers.borrow_mut(j);
