@@ -1,6 +1,7 @@
 use corundum::default::*;
+use corundum::open_flags::*;
 
-type P = BuddyAlloc;
+type P = Allocator;
 
 fn main() {
     use std::env;
