@@ -16,7 +16,7 @@ pub use vbtree::*;
 
 use corundum::default::*;
 
-pub type P = BuddyAlloc;
+pub type P = Allocator;
 
 pub trait Map<K, V> {
     fn clear(&self) {}
@@ -28,7 +28,7 @@ pub trait Map<K, V> {
     fn foreach<F: Copy + Fn(&K, &V) -> bool>(&self, _f: F) -> bool {
         unimplemented!()
     }
-    fn lookup(&self, _key: K) -> Option<&V> {
-        None
+    fn lookup(&self, _key: K) -> bool {
+        false
     }
 }
