@@ -1,10 +1,11 @@
 use std::time::Instant;
+use corundum::open_flags::*;
 
 fn main() {
     use corundum::default::*;
     use std::env;
 
-    type P = BuddyAlloc;
+    type P = Allocator;
 
     struct Root {
         list: PVec<PCell<i32>>
