@@ -1,3 +1,5 @@
+use std::marker::PhantomData;
+use std::panic::RefUnwindSafe;
 use crate::cell::{RootCell, RootObj};
 use crate::result::Result;
 use crate::stm::*;
@@ -9,7 +11,7 @@ use std::ops::Range;
 use std::panic::UnwindSafe;
 use std::path::Path;
 use std::thread::ThreadId;
-use std::{alloc::Layout, mem, ptr};
+use std::{mem, ptr};
 
 /// Default pool memory size to be used while creating a new pool
 pub const DEFAULT_POOL_SIZE: u64 = 8 * 1024 * 1024;
@@ -86,7 +88,7 @@ pub mod open_flags {
     pub const O_READINFO: u32 = u32::MAX;
 }
 
-pub use open_flags::*;
+use open_flags::*;
 
 /// Shows that the pool has a root object
 pub const FLAG_HAS_ROOT: u64 = 0x0000_0001;
@@ -96,7 +98,8 @@ pub const FLAG_HAS_ROOT: u64 = 0x0000_0001;
 macro_rules! static_inner {
     ($id:ident, $inner:ident, $body:block) => {
         unsafe {
-            if let Some($inner) = &mut $id {
+            if let Some($inner) = $id {
+                let $inner = &mut *$inner;
                 $body
             } else {
                 panic!("No memory pool is open or the root object is moved to a transaction. Try cloning the root object instead of moving it to a transaction.");
@@ -117,7 +120,7 @@ macro_rules! static_inner {
 /// To implement a new memory pool, you should define a new type with static
 /// values, that implements `MemPool`. You may redefine the default allocator as
 /// a new pool using [`pool!()`] which creates a pool module and generates the
-/// necessary code segments of type [`BuddyAlloc`].
+/// necessary code segments of type [`Allocator`].
 ///
 /// # Examples
 /// The following example shows how to use `MemPool` to track allocations of a
@@ -166,8 +169,8 @@ macro_rules! static_inner {
 /// // Declare p2 module
 /// pool!(p2);
 /// 
-/// let _pool1 = p1::BuddyAlloc::open_no_root("p1.pool", O_CF).unwrap();
-/// let _pool2 = p2::BuddyAlloc::open_no_root("p2.pool", O_CF).unwrap();
+/// let _pool1 = p1::Allocator::open_no_root("p1.pool", O_CF).unwrap();
+/// let _pool2 = p2::Allocator::open_no_root("p2.pool", O_CF).unwrap();
 /// 
 /// transaction(|j| {
 ///     // Create a Pbox object in p1
@@ -191,8 +194,8 @@ macro_rules! static_inner {
 /// RAII. They internally use the unsafe methods.
 /// 
 /// [`pool!()`]: ./default/macro.pool.html
-/// [`BuddyAlloc`]: ../default/struct.BuddyAlloc.html
-pub unsafe trait MemPool
+/// [`Allocator`]: ../default/struct.Allocator.html
+pub unsafe trait MemPoolTraits
 where
     Self: 'static + Sized,
 {
@@ -215,7 +218,7 @@ where
     ///   * O_CFNE: create and format a memory pool file only if not exists
     /// 
     /// See [`open_flags`](./open_flags/index.html) for more options.
-    fn open_no_root(_path: &str, _flags: u32) -> Result<Self> {
+    fn open_no_root(_path: &str, _flags: u32) -> Result<PoolGuard<Self>> {
         unimplemented!()
     }
 
@@ -256,7 +259,7 @@ where
     /// ```
     /// use corundum::default::*;
     ///
-    /// let root = BuddyAlloc::open::<i32>("foo.pool", O_CF).unwrap();
+    /// let root = Allocator::open::<i32>("foo.pool", O_CF).unwrap();
     ///
     /// assert_eq!(*root, i32::default());
     /// ```
@@ -271,7 +274,7 @@ where
     ///
     /// type Root = Prc<PCell<i32>>;
     ///
-    /// let root = BuddyAlloc::open::<Root>("foo.pool", O_CF).unwrap();
+    /// let root = Allocator::open::<Root>("foo.pool", O_CF).unwrap();
     ///
     /// let data = root.get();
     ///
@@ -297,7 +300,7 @@ where
     ///
     /// type Root = Parc<PMutex<i32>>;
     ///
-    /// let root = BuddyAlloc::open::<Root>("foo.pool", O_CF).unwrap();
+    /// let root = Allocator::open::<Root>("foo.pool", O_CF).unwrap();
     ///
     /// let mut threads = vec!();
     ///
@@ -334,10 +337,10 @@ where
     /// [`PCell`]: ./default/type.PCell.html
     /// [`PRefCell`]: ./default/type.PRefCell.html
     /// [`PMutex`]: ./default/type.PMutex.html
-    fn open<'a, U: 'a + PSafe + RootObj<Self>>(
+    fn open<'a, U: 'a + PSafe + RootObj<Self>> (
         _path: &str,
         _flags: u32,
-    ) -> Result<RootCell<'a, U, Self>> {
+    ) -> Result<RootCell<'a, U, Self>> where Self: MemPool {
         unimplemented!()
     }
 
@@ -409,7 +412,7 @@ where
         let _perf = crate::stat::Measure::<Self>::Deref(std::time::Instant::now());
 
         #[cfg(any(feature = "check_access_violation", debug_assertions))]
-        assert!( Self::allocated(off, 1), "Bad address (0x{:x})", off );
+        assert!( Self::allocated(off, 1), "Access Violation (0x{:x})", off );
 
         utils::read_addr(Self::start() + off)
     }
@@ -426,7 +429,7 @@ where
         let _perf = crate::stat::Measure::<Self>::Deref(std::time::Instant::now());
 
         #[cfg(any(feature = "check_access_violation", debug_assertions))]
-        assert!( Self::allocated(off, 1), "Bad address (0x{:x})", off );
+        assert!( Self::allocated(off, 1), "Access Violation (0x{:x})", off );
 
         utils::read_addr(Self::start() + off)
     }
@@ -441,7 +444,7 @@ where
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<Self>::Deref(std::time::Instant::now());
 
-        if len == 0 {
+        if off == u64::MAX {
             &[]
         } else {
             let ptr = utils::read_addr(Self::start() + off);
@@ -449,10 +452,10 @@ where
 
             #[cfg(any(feature = "check_access_violation", debug_assertions))]
             assert!(
-                Self::allocated(off, mem::size_of::<T>() * len),
-                "Bad address (0x{:x}..0x{:x})",
+                Self::allocated(off, mem::size_of::<T>().max(1) * len),
+                "Access Violation (0x{:x}..0x{:x})",
                 off,
-                off + (mem::size_of::<T>() * len) as u64 - 1
+                off.checked_add((mem::size_of::<T>().max(1) * len) as u64 - 1).unwrap_or_default()
             );
 
             res
@@ -469,7 +472,7 @@ where
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<Self>::Deref(std::time::Instant::now());
 
-        if len == 0 {
+        if off == u64::MAX {
             &mut []
         } else {
             let ptr = utils::read_addr(Self::start() + off);
@@ -477,10 +480,10 @@ where
 
             #[cfg(any(feature = "check_access_violation", debug_assertions))]
             assert!(
-                Self::allocated(off, mem::size_of::<T>() * len),
-                "Bad address (0x{:x}..0x{:x})",
+                Self::allocated(off, mem::size_of::<T>().max(1) * len),
+                "Access Violation (0x{:x}..0x{:x})",
                 off,
-                off + (mem::size_of::<T>() * len) as u64 - 1
+                off + (mem::size_of::<T>().max(1) * len) as u64 - 1
             );
 
             res
@@ -493,7 +496,7 @@ where
         if Self::allocated(off, mem::size_of::<T>()) {
             Ok(Self::get_unchecked(off))
         } else {
-            Err(format!("Bad address (0x{:x})", off))
+            Err(format!("Access Violation (0x{:x})", off))
         }
     }
 
@@ -503,7 +506,7 @@ where
         if Self::allocated(off, mem::size_of::<T>()) {
             Ok(Self::get_mut_unchecked(off))
         } else {
-            Err(format!("Bad address (0x{:x})", off))
+            Err(format!("Access Violation (0x{:x})", off))
         }
     }
 
@@ -637,7 +640,7 @@ where
     /// 
     /// ```
     /// # use corundum::default::*;
-    /// # type P = BuddyAlloc;
+    /// # type P = Allocator;
     /// # let _p = P::open_no_root("foo.pool", O_CF).unwrap();
     /// unsafe {
     ///     let (ptr, _, _, z) = P::pre_alloc(8);
@@ -661,7 +664,7 @@ where
     /// 
     /// ```
     /// # use corundum::default::*;
-    /// # type P = BuddyAlloc;
+    /// # type P = Allocator;
     /// # let _p = P::open_no_root("foo.pool", O_CF).unwrap();
     /// unsafe {
     ///     let (ptr, _, _) = P::alloc(8);
@@ -695,7 +698,7 @@ where
     /// 
     /// ```
     /// # use corundum::default::*;
-    /// # type P = BuddyAlloc;
+    /// # type P = Allocator;
     /// # let _p = P::open_no_root("foo.pool", O_CF).unwrap();
     /// unsafe {
     ///     // Prepare an allocation. The allocation is not durable yet. In case
@@ -790,7 +793,7 @@ where
     }
 
     /// Allocates new memory and then places `x` into it with `DropOnFailure` log
-    unsafe fn new<'a, T: PSafe + 'a>(x: T, j: &Journal<Self>) -> &'a mut T {
+    unsafe fn new<'a, T: PSafe + 'a>(x: T, j: &Journal<Self>) -> &'a mut T where Self: MemPool {
         debug_assert!(mem::size_of::<T>() != 0, "Cannot allocated ZST");
 
         let mut log = Log::drop_on_failure(u64::MAX, 1, j);
@@ -801,7 +804,7 @@ where
     }
 
     /// Allocates a new slice and then places `x` into it with `DropOnAbort` log
-    unsafe fn new_slice<'a, T: PSafe + 'a>(x: &'a [T], journal: &Journal<Self>) -> &'a mut [T] {
+    unsafe fn new_slice<'a, T: PSafe + 'a>(x: &'a [T], journal: &Journal<Self>) -> &'a mut [T] where Self: MemPool {
         debug_assert!(mem::size_of::<T>() != 0, "Cannot allocate ZST");
         debug_assert!(!x.is_empty(), "Cannot allocate empty slice");
 
@@ -814,7 +817,7 @@ where
 
     /// Allocates new memory and then copies `x` into it with `DropOnFailure` log
     unsafe fn new_copy<'a, T: 'a>(x: &T, j: &Journal<Self>) -> &'a mut T 
-    where T: ?Sized {
+    where T: ?Sized, Self: MemPool {
         let s = mem::size_of_val(x);
         debug_assert!(s != 0, "Cannot allocated ZST");
 
@@ -827,11 +830,11 @@ where
         std::ptr::copy_nonoverlapping(x as *const T as *const u8, p, s);
         log.set(off, len, z);
         Self::perform(z);
-        utils::read(p)
+        &mut *utils::read(p)
     }
 
     /// Allocates new memory and then copies `x` into it with `DropOnFailure` log
-    unsafe fn new_copy_slice<'a, T: 'a>(x: &[T], j: &Journal<Self>) -> &'a mut [T] {
+    unsafe fn new_copy_slice<'a, T: 'a>(x: &[T], j: &Journal<Self>) -> &'a mut [T] where Self: MemPool {
         let s = mem::size_of_val(x);
         debug_assert!(s != 0, "Cannot allocated ZST");
 
@@ -844,7 +847,7 @@ where
         std::ptr::copy_nonoverlapping(x as *const [T] as *const u8, p, s);
         log.set(off, len, z);
         Self::perform(z);
-        utils::read(p)
+        &mut *utils::read(p)
     }
 
     /// Allocates new memory and then places `x` into it without realizing the allocation
@@ -857,7 +860,7 @@ where
             panic!("Memory exhausted");
         }
         Self::drop_on_failure(off, len, z);
-        let p = utils::read(raw);
+        let p = &mut *utils::read(raw);
         mem::forget(ptr::replace(p, x));
         (p, off, size, z)
     }
@@ -866,7 +869,7 @@ where
     unsafe fn atomic_new_slice<'a, T: 'a + PSafe>(x: &'a [T]) -> (&'a mut [T], u64, usize, usize) {
         log!(Self, White, "ALLOC", "TYPE: [{}; {}]", std::any::type_name::<T>(), x.len());
 
-        let (ptr, off, size, z) = Self::pre_alloc(Layout::for_value(x).size());
+        let (ptr, off, size, z) = Self::pre_alloc(mem::size_of_val(x));
         if ptr.is_null() {
             panic!("Memory exhausted");
         }
@@ -874,7 +877,7 @@ where
         ptr::copy_nonoverlapping(
             x as *const _ as *const u8,
             ptr,
-            x.len() * mem::size_of::<T>(),
+            x.len() * mem::size_of::<T>().max(1),
         );
         (
             std::slice::from_raw_parts_mut(ptr.cast(), x.len()),
@@ -885,7 +888,7 @@ where
     }
 
     /// Allocates new memory without copying data
-    unsafe fn new_uninit<'a, T: PSafe + 'a>(j: &Journal<Self>) -> &'a mut T {
+    unsafe fn new_uninit<'a, T: PSafe + 'a>(j: &Journal<Self>) -> &'a mut T where Self: MemPool {
         let mut log = Log::drop_on_failure(u64::MAX, 1, j);
         let (p, off, size, z) = Self::atomic_new_uninit();
         log.set(off, size, z);
@@ -894,7 +897,7 @@ where
     }
 
     /// Allocates new memory without copying data
-    unsafe fn new_uninit_for_layout(size: usize, journal: &Journal<Self>) -> *mut u8 {
+    unsafe fn new_uninit_for_layout(size: usize, journal: &Journal<Self>) -> *mut u8 where Self: MemPool {
         log!(Self, White, "ALLOC", "{:?}", size);
 
         let mut log = Log::drop_on_abort(u64::MAX, 1, journal);
@@ -915,7 +918,7 @@ where
             panic!("Memory exhausted");
         }
         Self::drop_on_failure(off, len, z);
-        (utils::read(ptr), off, len, z)
+        (&mut *utils::read(ptr), off, len, z)
     }
 
     /// Allocates new memory for value `x`
@@ -924,11 +927,11 @@ where
         if raw.0.is_null() {
             panic!("Memory exhausted");
         }
-        utils::read(raw.0)
+        &mut *utils::read(raw.0)
     }
 
     /// Creates a `DropOnCommit` log for the value `x`
-    unsafe fn free<'a, T: PSafe + ?Sized>(x: &mut T) {
+    unsafe fn free<'a, T: PSafe + ?Sized>(x: &mut T) where Self: MemPool {
         // std::ptr::drop_in_place(x);
         let off = Self::off_unchecked(x);
         let len = mem::size_of_val(x);
@@ -940,13 +943,13 @@ where
     }
 
     /// Creates a `DropOnCommit` log for the value `x`
-    unsafe fn free_slice<'a, T: PSafe>(x: &mut [T]) {
+    unsafe fn free_slice<'a, T: PSafe>(x: &[T]) where Self: MemPool {
         // eprintln!("FREEING {} of size {}", x as *mut u8 as u64, len);
         if x.len() > 0 {
             let off = Self::off_unchecked(x);
             Log::drop_on_commit(
                 off,
-                x.len() * mem::size_of::<T>(),
+                x.len() * mem::size_of::<T>().max(1),
                 &*Journal::<Self>::current(true).unwrap().0,
             );
         }
@@ -960,7 +963,7 @@ where
     }
 
     /// Drops a `journal` from memory
-    unsafe fn drop_journal(_journal: &mut Journal<Self>) { }
+    unsafe fn drop_journal(_journal: &mut Journal<Self>) where Self: MemPool { }
 
     /// Returns a reference to the offset of the first journal
     unsafe fn journals_head() -> &'static u64 { unimplemented!() }
@@ -986,7 +989,7 @@ where
     ///
     #[inline]
     #[track_caller]
-    unsafe fn commit() {
+    unsafe fn commit() where Self: MemPool {
         // Self::discard(crate::ll::cpu());
         if let Some(journal) = Journal::<Self>::current(false) {
             *journal.1 -= 1;
@@ -995,8 +998,14 @@ where
                 log!(Self, White, "COMMIT", "JRNL: {:?}", journal.0);
 
                 let journal = as_mut(journal.0);
-                journal.commit();
-                journal.clear();
+                journal.commit(
+                    #[cfg(feature = "check_double_free")]
+                    &mut *Self::dealloc_history()
+                );
+                journal.clear(
+                    #[cfg(feature = "check_double_free")]
+                    &mut *Self::dealloc_history()
+                );
             }
         }
     }
@@ -1011,7 +1020,7 @@ where
     ///
     /// This function is for internal use and should not be called elsewhere.
     ///
-    unsafe fn commit_no_clear() {
+    unsafe fn commit_no_clear() where Self: MemPool {
         // Self::discard(crate::ll::cpu());
         if let Some(journal) = Journal::<Self>::current(false) {
             *journal.1 -= 1;
@@ -1019,7 +1028,10 @@ where
             if *journal.1 == 0 {
                 log!(Self, White, "COMMIT_NC", "JRNL: {:?}", journal.0);
 
-                as_mut(journal.0).commit();
+                as_mut(journal.0).commit(
+                    #[cfg(feature = "check_double_free")]
+                    &mut *Self::dealloc_history()
+                );
             }
         }
     }
@@ -1034,14 +1046,17 @@ where
     ///
     /// This function is for internal use and should not be called elsewhere.
     ///
-    unsafe fn clear() {
+    unsafe fn clear() where Self: MemPool {
         if let Some(journal) = Journal::<Self>::current(false) {
             *journal.1 -= 1;
 
             if *journal.1 == -1 {
                 log!(Self, White, "CLEAR", "JRNL: {:?}", journal.0);
 
-                as_mut(journal.0).clear();
+                as_mut(journal.0).clear(
+                    #[cfg(feature = "check_double_free")]
+                    &mut *Self::dealloc_history()
+                );
             }
         }
     }
@@ -1057,7 +1072,7 @@ where
     ///
     /// This function is for internal use and should not be called elsewhere.
     ///
-    unsafe fn rollback() -> bool {
+    unsafe fn rollback() -> bool where Self: MemPool {
         // Self::discard(crate::ll::cpu());
         if let Some(journal) = Journal::<Self>::current(false) {
             *journal.1 -= 1;
@@ -1066,8 +1081,14 @@ where
                 log!(Self, White, "ROLLBACK", "JRNL: {:?}", journal.0);
 
                 let journal = as_mut(journal.0);
-                journal.rollback();
-                journal.clear();
+                journal.rollback(
+                    #[cfg(feature = "check_double_free")]
+                    &mut *Self::dealloc_history()
+                );
+                journal.clear(
+                    #[cfg(feature = "check_double_free")]
+                    &mut *Self::dealloc_history()
+                );
                 return true;
             } else {
                 // Propagate the panic to the upper transactions
@@ -1087,18 +1108,25 @@ where
     ///
     /// This function is for internal use and should not be called elsewhere.
     ///
-    unsafe fn rollback_no_clear() {
+    unsafe fn rollback_no_clear() where Self: MemPool {
         if let Some(journal) = Journal::<Self>::current(false) {
             *journal.1 -= 1;
 
             if *journal.1 == 0 {
                 log!(Self, White, "ROLLBACK_NC", "JRNL: {:?}", journal.0);
 
-                as_mut(journal.0).rollback();
+                as_mut(journal.0).rollback(
+                    #[cfg(feature = "check_double_free")]
+                    &mut *Self::dealloc_history()
+                );
             }
         }
     }
 
+    unsafe fn dealloc_history() -> *mut std::collections::HashSet<u64> {
+        unimplemented!()
+    }
+
     /// Executes commands atomically with respect to system crashes
     /// 
     /// The `transaction` function takes a closure with one argument of type
@@ -1122,12 +1150,12 @@ where
     /// ```
     /// use corundum::default::*;
     /// 
-    /// type P = BuddyAlloc;
+    /// type P = Allocator;
     /// 
     /// let root = P::open::<PCell<i32>>("foo.pool", O_CF).unwrap();
     /// 
     /// let old = root.get();
-    /// let new = BuddyAlloc::transaction(|j| {
+    /// let new = Allocator::transaction(|j| {
     ///     root.set(root.get() + 1, j);
     ///     root.get()
     /// }).unwrap();
@@ -1145,7 +1173,7 @@ where
     fn transaction<T, F: FnOnce(&'static Journal<Self>) -> T>(body: F) -> Result<T>
     where
         F: TxInSafe + UnwindSafe,
-        T: TxOutSafe,
+        T: TxOutSafe, Self: alloc::pool::MemPool
     {
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<Self>::Transaction;
@@ -1231,12 +1259,48 @@ where
     /// Prints memory information
     fn print_info() {}
 
-    #[cfg(feature = "stat_footprint")]
     fn stat_footprint() -> usize {
-        0
+        if cfg!(feature = "stat_footprint") {
+            0
+        } else {
+            unimplemented!()
+        }
     }
 }
 
+pub struct PoolGuard<P: MemPoolTraits>(pub PhantomData<P>);
+
+impl<P: MemPoolTraits> PoolGuard<P> {
+    pub fn new() -> Self {
+        Self(PhantomData)
+    }
+}
+
+impl<P: MemPoolTraits> Drop for PoolGuard<P> {
+    fn drop(&mut self) {
+        unsafe {
+            P::close().unwrap();
+        }
+
+        crate::__cfg_stat_perf!({
+            eprintln!("{}", crate::stat::report());
+        });
+    }
+}
+
+pub unsafe trait MemPool: 
+    'static + 
+    MemPoolTraits + 
+    Sized + 
+    Default + 
+    Clone +
+    Copy +
+    PSafe + 
+    TxInSafe + 
+    LooseTxInUnsafe + 
+    RefUnwindSafe + 
+    UnwindSafe {}
+
 pub(crate) fn create_file(filename: &str, size: u64) -> Result<()> {
     let file = OpenOptions::new().write(true).create(true).open(filename);
     if file.is_err() {
@@ -1252,16 +1316,18 @@ pub(crate) fn create_file(filename: &str, size: u64) -> Result<()> {
 
 #[cfg(test)]
 mod test {
+    use crate::open_flags::*;
+    use crate::alloc::pool::MemPoolTraits;
     use crate::default::*;
 
     #[test]
     #[ignore]
     fn nested_transactions() {
-        let _image = BuddyAlloc::open_no_root("nosb.pool", O_CFNE);
-        if let Err(e) = BuddyAlloc::transaction(|_| {
-            let _ = BuddyAlloc::transaction(|_| {
-                let _ = BuddyAlloc::transaction(|_| {
-                    let _ = BuddyAlloc::transaction(|_| {
+        let _image = Allocator::open_no_root("nosb.pool", O_CFNE);
+        if let Err(e) = Allocator::transaction(|_| {
+            let _ = Allocator::transaction(|_| {
+                let _ = Allocator::transaction(|_| {
+                    let _ = Allocator::transaction(|_| {
                         println!("should print");
                         panic!("intentional");
                     });
