@@ -1,5 +1,3 @@
-use crate::alloc::MemPool;
-use crate::alloc::PmemUsage;
 use crate::*;
 use std::marker::PhantomData;
 use std::ops::Index;
@@ -7,9 +5,9 @@ use std::ops::Index;
 /// A persistent fat pointer with offset and capacity
 #[derive(Eq)]
 pub struct Slice<T: PSafe, A: MemPool> {
-    off: u64,
-    cap: usize,
-    dummy: [A; 0],
+    pub(crate) off: u64,
+    pub(crate) cap: usize,
+    dummy: [*const A; 0],
     marker: PhantomData<[T]>,
 }
 
@@ -35,11 +33,20 @@ impl<T: PSafe, A: MemPool> Slice<T, A> {
         }
     }
 
-    /// Sets the capacity to zero
-    pub fn empty() -> Self {
+    /// Create an empty slice
+    pub fn null() -> Self {
         Self::from_off_cap(u64::MAX, 0)
     }
 
+    /// Creates a new fat pointer given a slice
+    pub unsafe fn from_raw_parts(x: *const u8, len: usize) -> Self {
+        if len == 0 {
+            Self::from_off_cap(u64::MAX, 0)
+        } else {
+            Self::from_off_cap(A::off_unchecked(x), len)
+        }
+    }
+
     /// Returns true if the capacity is zero
     #[inline]
     pub fn is_empty(&self) -> bool {
@@ -119,26 +126,18 @@ impl<T: PSafe, A: MemPool> Slice<T, A> {
 
     /// Converts the fat pointer into a slice of type `&[T]`
     #[inline]
-    pub unsafe fn as_slice(&self) -> &[T] {
+    pub fn as_slice(&self) -> &[T] {
         Self::to_slice(self.off, self.cap)
     }
 
     #[inline]
     pub(crate) fn to_slice<'a>(off: u64, len: usize) -> &'a [T] {
-        if len == 0 {
-            &mut []
-        } else {
-            unsafe { A::deref_slice_unchecked(off, len) }
-        }
+        unsafe { A::deref_slice_unchecked(off, len) }
     }
 
     #[inline]
     pub(crate) fn as_slice_mut(&mut self) -> &mut [T] {
-        if self.cap == 0 {
-            &mut []
-        } else {
-            unsafe { A::deref_slice_unchecked_mut(self.off, self.cap) }
-        }
+        unsafe { A::deref_slice_unchecked_mut(self.off, self.cap) }
     }
 
     /// Divides one slice into two at an index.
@@ -188,12 +187,11 @@ impl<T: PSafe, A: MemPool> Slice<T, A> {
     /// 
     pub unsafe fn dup(&self) -> Slice<T, A> {
         if self.is_empty() {
-            Self::empty()
+            Self::null()
         } else {
             let slice = self.as_slice();
-            let (dst, off, len, z) = A::atomic_new_slice(slice);
+            let (_, off, len, z) = A::atomic_new_slice(slice);
             A::perform(z);
-            crate::ll::persist(dst, self.cap, true);
             Self::from_off_cap(off, len)
         }
     }
@@ -220,11 +218,30 @@ impl<A: MemPool, T: PSafe> From<&mut [T]> for Slice<T, A> {
     }
 }
 
-impl<A: MemPool + Copy, T: PSafe + Copy> Copy for Slice<T, A> {}
+impl<A: MemPool, T: PSafe + Copy> Copy for Slice<T, A> {}
 
 impl<A: MemPool, T: PSafe> Clone for Slice<T, A> {
     fn clone(&self) -> Self {
-        Self::from_off_cap(self.off, self.cap)
+        unsafe {
+            if self.cap == 0 {
+                Self::null()
+            } else {
+                let j = Journal::<A>::current(false).expect("`Slice::clone()` is transactional");
+                self.pclone(&*j.0)
+            }
+        }
+    }
+}
+
+impl<A: MemPool, T: PSafe> PClone<A> for Slice<T, A> {
+    fn pclone(&self, j: &Journal<A>) -> Self {
+        unsafe {
+            if self.cap == 0 {
+                Self::null()
+            } else {
+                Self::new(A::new_copy_slice(self.as_slice(), j))
+            }
+        }
     }
 }
 
@@ -236,7 +253,7 @@ impl<A: MemPool, T: PSafe> PmemUsage for Slice<T, A> {
 
 impl<A: MemPool, T: PSafe> Default for Slice<T, A> {
     fn default() -> Self {
-        Slice::empty()
+        Slice::null()
     }
 }
 
