@@ -7,6 +7,8 @@ use crate::*;
 use std::collections::HashMap;
 use std::fmt::{self, Debug, Formatter};
 
+#[cfg(feature = "check_double_free")]
+use std::collections::HashSet;
 
 #[cfg(all(feature = "use_pspd", feature = "use_vspd"))]
 compile_error!("Cannot use both volatile and persistent scratchpad");
@@ -98,7 +100,7 @@ impl<A: MemPool> Page<A> {
         #[cfg(feature = "use_ntstore")] unsafe {
             std::intrinsics::nontemporal_store(&mut self.logs[self.len], Log::new(log, notifier));
         }
-        persist(&self.logs[self.len], std::mem::size_of::<Log<A>>(), false);
+        persist_with_log::<_,A>(&self.logs[self.len], std::mem::size_of::<Log<A>>(), false);
 
         let log = unsafe { Ptr::new_unchecked(&self.logs[self.len]) };
         self.len += 1;
@@ -116,9 +118,21 @@ impl<A: MemPool> Page<A> {
         }
     }
 
-    unsafe fn commit(&mut self) {
+    unsafe fn commit_data(&mut self) {
         for i in 0..self.len {
-            self.logs[i].commit();
+            self.logs[i].commit_data();
+        }
+    }
+
+    unsafe fn commit_dealloc(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
+        for i in 0..self.len {
+            self.logs[i].commit_dealloc(
+                #[cfg(feature = "check_double_free")]
+                check_double_free
+            );
         }
     }
 
@@ -126,14 +140,29 @@ impl<A: MemPool> Page<A> {
         for i in 0..self.len {
             self.logs[self.len - i - 1].rollback();
         }
+    }
+
+    unsafe fn rollback_dealloc(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         for i in 0..self.len {
-            self.logs[i].rollback_drop_on_abort();
+            self.logs[i].rollback_drop_on_abort(
+                #[cfg(feature = "check_double_free")]
+                check_double_free
+            );
         }
     }
 
-    unsafe fn recover(&mut self, rollback: bool) {
+    unsafe fn recover(&mut self, rollback: bool, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         for i in 0..self.len {
-            self.logs[self.len - i - 1].recover(rollback);
+            self.logs[self.len - i - 1].recover(rollback, 
+                #[cfg(feature = "check_double_free")]
+                check_double_free
+            );
         }
     }
 
@@ -143,9 +172,15 @@ impl<A: MemPool> Page<A> {
         self.logs = [Default::default(); PAGE_LOG_SLOTS];
     }
 
-    unsafe fn clear(&mut self) {
+    unsafe fn clear(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         for i in self.head..self.len {
-            self.logs[i].clear();
+            self.logs[i].clear(
+                #[cfg(feature = "check_double_free")]
+                check_double_free
+            );
             self.head += 1;
         }
 
@@ -205,7 +240,7 @@ impl<A: MemPool> Journal<A> {
     /// Sets a flag
     pub unsafe fn set(&mut self, flag: u64) {
         self.flags |= flag;
-        persist_obj(&self.flags, true);
+        persist_obj_with_log::<_,A>(&self.flags, true);
     }
 
     /// Resets a flag
@@ -386,7 +421,10 @@ impl<A: MemPool> Journal<A> {
     }
 
     /// Commits all logs in the journal
-    pub unsafe fn commit(&mut self) {
+    pub unsafe fn commit(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         #[cfg(any(feature = "use_pspd", feature = "use_vspd"))] {
             self.spd.commit();
         }
@@ -397,7 +435,15 @@ impl<A: MemPool> Journal<A> {
         }
         let mut curr = self.pages;
         while let Some(page) = curr.as_option() {
-            page.commit();
+            page.commit_data();
+            curr = page.next;
+        }
+        let mut curr = self.pages;
+        while let Some(page) = curr.as_option() {
+            page.commit_dealloc(
+                #[cfg(feature = "check_double_free")]
+                check_double_free
+            );
             curr = page.next;
         }
         sfence();
@@ -405,7 +451,10 @@ impl<A: MemPool> Journal<A> {
     }
 
     /// Reverts all changes
-    pub unsafe fn rollback(&mut self) {
+    pub unsafe fn rollback(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         #[cfg(any(feature = "use_pspd", feature = "use_vspd"))] {
             self.spd.rollback();
         }
@@ -419,12 +468,23 @@ impl<A: MemPool> Journal<A> {
             page.rollback();
             curr = page.next;
         }
+        let mut curr = self.pages;
+        while let Some(page) = curr.as_option() {
+            page.rollback_dealloc(
+                #[cfg(feature = "check_double_free")]
+                check_double_free
+            );
+            curr = page.next;
+        }
         sfence();
         self.set(JOURNAL_COMMITTED);
     }
 
     /// Recovers from a crash or power failure
-    pub unsafe fn recover(&mut self) {
+    pub unsafe fn recover(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         let mut curr = self.pages;
         while let Some(page) = curr.as_option() {
             page.notify();
@@ -442,7 +502,10 @@ impl<A: MemPool> Journal<A> {
                 }
             }
             while let Some(page) = curr.as_option() {
-                page.recover(rollback);
+                page.recover(rollback, 
+                    #[cfg(feature = "check_double_free")]
+                    check_double_free
+                );
                 curr = page.next;
             }
             self.set(JOURNAL_COMMITTED);
@@ -450,7 +513,10 @@ impl<A: MemPool> Journal<A> {
     }
 
     /// Clears all logs and drops itself from the memory pool
-    pub unsafe fn clear(&mut self) {
+    pub unsafe fn clear(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         #[cfg(any(feature = "use_pspd", feature = "use_vspd"))] {
             self.spd.clear();
         }
@@ -458,7 +524,10 @@ impl<A: MemPool> Journal<A> {
         {
             let mut page = self.pages.as_option();
             while let Some(p) = page {
-                p.clear();
+                p.clear(
+                    #[cfg(feature = "check_double_free")]
+                    check_double_free
+                );
                 page = p.next.as_option();
             }
             self.current = self.pages;
@@ -467,7 +536,10 @@ impl<A: MemPool> Journal<A> {
         #[cfg(not(feature = "pin_journals"))] {
             while let Some(page) = self.pages.as_option() {
                 let nxt = page.next;
-                page.clear();
+                page.clear(
+                    #[cfg(feature = "check_double_free")]
+                    check_double_free
+                );
                 let z = A::pre_dealloc(page.as_mut_ptr() as *mut u8, std::mem::size_of::<Page<A>>());
                 A::log64(A::off_unchecked(self.pages.off_ref()), nxt.off(), z);
                 A::perform(z);
@@ -559,7 +631,7 @@ impl<A: MemPool> Journal<A> {
                     let id = self.sec_id;
                     self.chaperon = [0; 64];
                     self.sec_id = 0;
-                    persist_obj(&self.sec_id, true);
+                    persist_obj_with_log::<_,A>(&self.sec_id, true);
                     c.finish(id as usize);
                 } else {
                     self.chaperon = [0; 64];
