@@ -8,6 +8,9 @@ use std::clone::Clone;
 use std::fmt::{self, Debug};
 use std::ptr;
 
+#[cfg(feature = "check_double_free")]
+use std::collections::HashSet;
+
 type Offset = u64;
 
 /// Log Types
@@ -208,7 +211,7 @@ impl<A: MemPool> Log<A> {
     /// ```
     /// # use corundum::default::*;
     /// # use corundum::stm::Log;
-    /// # type P = BuddyAlloc;
+    /// # type P = Allocator;
     /// # let _p = P::open_no_root("foo.pool", O_CF).unwrap();
     /// P::transaction(|j| unsafe {
     ///     // Create a neutral high-level log to drop the allocation on failure.
@@ -296,17 +299,20 @@ fn dump_data<A: MemPool>(tag: &str, off: u64, len: usize) {
     use term_painter::Color::*;
     use term_painter::ToStyle;
 
-    print!("{}", BrightBlue.paint(format!("{:>19}  ", tag)));
-    for i in 0..len {
-        let d = unsafe { A::get_unchecked::<u8>(off + i as u64) };
-        print!("{}", BrightBlue.paint(format!("{:02x} ", *d)));
-        if i % 16 == 15 && i+1 < len {
-            println!();
-            print!("{:>21}", " ");
+    if *crate::utils::VERBOSE {
+        print!("{:<8} {}", A::name().to_owned() + ":", 
+            BrightBlue.paint(format!("{:>10}  ", tag)));
+        for i in 0..len {
+            let d = unsafe { A::get_unchecked::<u8>(off + i as u64) };
+            print!("{}", BrightBlue.paint(format!("{:02x} ", *d)));
+            if i % 16 == 15 && i+1 < len {
+                println!();
+                print!("{:>21}", " ");
+            }
         }
+    
+        println!();
     }
-
-    println!();
 }
 
 impl<A: MemPool> Log<A> {
@@ -364,8 +370,8 @@ impl<A: MemPool> Log<A> {
 
             //     Self::create_impl(log.off(), pointer.off(), len, journal, notifier)
             // } else {
-                crate::ll::persist_obj(log.as_ref(), false);
-                Self::create_impl(pointer.off(), log.off(), len, journal, notifier)
+            crate::ll::persist_obj_with_log::<_,A>(log.as_ref(), false);
+            Self::create_impl(pointer.off(), log.off(), len, journal, notifier)
             // }
         }
     }
@@ -397,8 +403,8 @@ impl<A: MemPool> Log<A> {
 
             let log = unsafe { slice.dup() };
 
-                crate::ll::persist_obj(log.as_ref(), false);
-                Self::create_impl(slice.off(), log.off(), len, journal, notifier)
+            crate::ll::persist_obj_with_log::<_,A>(log.as_ref(), false);
+            Self::create_impl(slice.off(), log.off(), len, journal, notifier)
             // }
         }
     }
@@ -514,7 +520,7 @@ impl<A: MemPool> Log<A> {
         debug_assert_ne!(*len, 0);
 
         if *log != u64::MAX && *src != u64::MAX {
-            log!(A, Magenta, "ROLLBACK", "FOR:         ({:>6}:{:<6}) = {:<6} DataLog({})",
+            log!(A, Magenta, "ROLLBACK", "FOR:         ({:>6x}:{:<6x}) = {:<6} DataLog({})",
                 *src, *src as usize + (len - 1), len, log   
             );
             #[cfg(feature = "verbose")] {
@@ -525,7 +531,7 @@ impl<A: MemPool> Log<A> {
                 let src = A::get_mut_unchecked::<u8>(*src);
                 let log = A::get_mut_unchecked::<u8>(*log);
                 ptr::copy_nonoverlapping(log, src, *len);
-                persist(log, *len, true);
+                persist_with_log::<_,A>(log, *len, false);
             }
                     
             #[cfg(feature = "check_allocator_cyclic_links")]
@@ -550,13 +556,22 @@ impl<A: MemPool> Log<A> {
         }
     }
 
-    pub(crate) unsafe fn rollback_drop_on_abort(&mut self) {
+    pub(crate) unsafe fn rollback_drop_on_abort(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<A>::RollbackLog(std::time::Instant::now());
 
         match &mut self.0 {
             DropOnAbort(src, len) => {
                 if *src != u64::MAX {
+                    #[cfg(feature = "check_double_free")] {
+                        if check_double_free.contains(&*src) {
+                            return;
+                        }
+                        check_double_free.insert(*src);
+                    }
                     let z = A::pre_dealloc(A::get_mut_unchecked(*src), *len);
                     A::log64(A::off_unchecked(src), u64::MAX, z);
                     A::perform(z);
@@ -570,7 +585,10 @@ impl<A: MemPool> Log<A> {
     }
 
     /// Recovers from the crash or power failure
-    pub(crate) unsafe fn recover(&mut self, rollback: bool) {
+    pub(crate) unsafe fn recover(&mut self, rollback: bool, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         match &mut self.0 {
             DataLog(src, log, layout) => {
                 if rollback {
@@ -587,7 +605,13 @@ impl<A: MemPool> Log<A> {
             DropOnFailure(src, len) => {
                 if rollback {
                     if *src != u64::MAX {
-                        debug_assert!(A::allocated(*src, 1), "Access Violation");
+                        #[cfg(feature = "check_double_free")] {
+                            if check_double_free.contains(&*src) {
+                                return;
+                            }
+                            check_double_free.insert(*src);
+                        }
+                        debug_assert!(A::allocated(*src, 1), "Access Violation (0x{:x})", *src);
                         let z = A::pre_dealloc(A::get_mut_unchecked(*src), *len);
                         A::log64(A::off_unchecked(src), u64::MAX, z);
                         A::perform(z);
@@ -626,7 +650,7 @@ impl<A: MemPool> Log<A> {
     }
 
     /// Commits changes
-    pub(crate) fn commit(&mut self) {
+    pub(crate) fn commit_data(&mut self) {
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<A>::CommitLog(std::time::Instant::now());
 
@@ -636,12 +660,31 @@ impl<A: MemPool> Log<A> {
 
                 #[cfg(all(not(feature = "no_flush_updates"), not(feature = "replace_with_log")))]
                 unsafe {
-                    persist::<u8>(A::get_mut_unchecked(*_src), *_len, false);
+                    persist_with_log::<u8,A>(A::get_mut_unchecked(*_src), *_len, false);
                 }
             }
+            _ => {}
+        }
+    }
+
+    /// Commits changes
+    pub(crate) fn commit_dealloc(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
+        #[cfg(feature = "stat_perf")]
+        let _perf = crate::stat::Measure::<A>::CommitLog(std::time::Instant::now());
+
+        match &mut self.0 {
             DropOnCommit(src, len) => {
                 if *src != u64::MAX {
                     unsafe {
+                        #[cfg(feature = "check_double_free")] {
+                            if check_double_free.contains(&*src) {
+                                return;
+                            }
+                            check_double_free.insert(*src);
+                        }
                         let z = A::pre_dealloc(A::get_mut_unchecked(*src), *len);
                         A::log64(A::off_unchecked(src), u64::MAX, z);
                         A::perform(z);
@@ -662,13 +705,22 @@ impl<A: MemPool> Log<A> {
     /// * If it is a [`UnlockOnCommit`](./enum.LogEnum.html#variant.UnlockOnCommit),
     /// it unlocks the mutex.
     /// 
-    pub unsafe fn clear(&mut self) {
+    pub unsafe fn clear(&mut self, 
+        #[cfg(feature = "check_double_free")]
+        check_double_free: &mut HashSet<u64>
+    ) {
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<A>::ClearLog(std::time::Instant::now());
 
         match &mut self.0 {
             DataLog(_src, log, len) => {
                 if *log != u64::MAX {
+                    #[cfg(feature = "check_double_free")] {
+                        if check_double_free.contains(&*log) {
+                            return;
+                        }
+                        check_double_free.insert(*log);
+                    }
                     log!(A, Magenta, "DEL LOG", "FOR:         ({:>6x}:{:<6x}) = {:<6} DataLog({})",
                         *_src, *_src as usize + (*len - 1), *len, log
                     );
