@@ -96,7 +96,8 @@ impl<T: PSafe, A: MemPool> Vec<T, A> {
             Self::empty()
         } else {
             let buf = unsafe { A::new_slice(x, journal) };
-            Self::from_off_len(unsafe { A::off_unchecked(buf) }, buf.len(), buf.len())
+            let offset = unsafe { A::off_unchecked(buf) };
+            Self::from_off_len(offset, buf.len(), buf.len())
         }
     }
 
@@ -363,6 +364,16 @@ impl<T: PSafe, A: MemPool> Vec<T, A> {
         Self::to_slice(self.off(), self.len)
     }
 
+    #[inline]
+    /// Returns a raw pointer to data if there is any
+    pub(crate) unsafe fn as_ptr(&self) -> *mut T {
+        if self.len == 0 {
+            std::ptr::null_mut()
+        } else {
+            A::get_mut_unchecked(self.off())
+        }
+    }
+
     #[inline]
     /// Consumes the vector and converts it into a slice.
     /// Since we should create a log of the context, this function is transactional
@@ -399,20 +410,12 @@ impl<T: PSafe, A: MemPool> Vec<T, A> {
 
     #[inline]
     fn to_slice<'a>(off: u64, len: usize) -> &'a [T] {
-        if len == 0 {
-            &mut []
-        } else {
-            unsafe { A::deref_slice_unchecked(off, len) }
-        }
+        unsafe { A::deref_slice_unchecked(off, len) }
     }
 
     #[inline]
     fn __to_slice_mut<'a>(off: u64, len: usize) -> &'a mut [T] {
-        if len == 0 || off == u64::MAX {
-            &mut []
-        } else {
-            unsafe { A::deref_slice_unchecked_mut(off, len) }
-        }
+        unsafe { A::deref_slice_unchecked_mut(off, len) }
     }
 
     #[inline]
@@ -468,7 +471,7 @@ impl<T: PSafe, A: MemPool> Vec<T, A> {
                     // of each individual item
                 }
                 if rem.is_empty() {
-                    self.buf = Slice::empty();
+                    self.buf = Slice::null();
                 } else {
                     let new = A::new_slice(rem, j);
                     self.buf = Slice::from_off_cap(A::off_unchecked(new), new.len());
@@ -1282,9 +1285,11 @@ impl<A: MemPool> Vec<u8, A> {
 
 #[cfg(test)]
 mod test {
+    use crate::RootObj;
     use crate::default::*;
+    use crate::open_flags::*;
 
-    type A = BuddyAlloc;
+    type A = Allocator;
 
     #[test]
     fn test_array() {
@@ -1363,7 +1368,7 @@ mod test {
     #[test]
     fn test_clear() {
         use crate::vec::Vec;
-        heap::Heap::transaction::<_, _>(|j| {
+        crate::heap::Heap::transaction::<_, _>(|j| {
             let mut vec = Vec::from_slice(&[1, 2, 3], j);
             vec.truncate(0);
             assert_eq!(vec, []);
